@@ -420,7 +420,8 @@ _hb_ot_shape_fallback_kern (const hb_ot_shape_plan_t *plan,
   hb_mask_t kern_mask = plan->map.get_1_mask (HB_DIRECTION_IS_HORIZONTAL (buffer->props.direction) ?
 					      HB_TAG ('k','e','r','n') : HB_TAG ('v','k','r','n'));
 
-  OT::hb_apply_context_t c (1, font, buffer, kern_mask, true/*auto_joiners*/);
+  OT::hb_apply_context_t c (1, font, buffer);
+  c.set_lookup_mask (kern_mask);
   c.set_lookup_props (OT::LookupFlag::IgnoreMarks);
 
   for (buffer->idx = 0; buffer->idx < count;)
